interface Window {
	getTime(): number;
	mozRequestAnimationFrame(): number;
	webkitRequestAnimationFrame(): number;
	createCanvas(width:number, height:number): HTMLCanvasElement;
}

window.getTime = (function() {
	if (window.performance && window.performance.now) {
		return function() {
			return window.performance.now();
		};
	} else if (window.performance && window.performance.webkitNow) {
		return function() {
			return window.performance.webkitNow();
		};
	} else {
		return Date.now;
	}
}());

window.requestAnimationFrame =
	window.requestAnimationFrame ||
	window.mozRequestAnimationFrame ||
	window.webkitRequestAnimationFrame ||
	window.msRequestAnimationFrame ||
	(function() {
		var lastTime = window.getTime();
		var frame = 1000 / 60;
		return function(func) {
			var currentTime = window.getTime();
			var _id = setTimeout(function() {
				func(window.getTime());
			}, Math.max(0, lastTime + frame - currentTime));
			lastTime = currentTime;
			return _id;
		};
	}());

window.createCanvas = function(width:number, height:number):HTMLCanvasElement {
	var canvas:HTMLCanvasElement = <HTMLCanvasElement>document.createElement("canvas");
	canvas.width = width;
	canvas.height = height;
	return canvas;
}
